@@ -53,7 +53,7 @@ DEFAULT_MODE = 0o710
 def copyanything(src, dst):
     try:
         copytree(src, dst)
-    except FileExistsError as exc:
+    except FileExistsError as e:  # noqa
         pass
     except OSError as exc:  # python >2.5
         if exc.errno == errno.ENOTDIR:
@@ -136,7 +136,7 @@ class SecretsEnvironment(object):
         if not (subdir is None and host is None):
             valid_subdir = 'a-zA-Z0-9_/'
             invalid_subdir = re.compile('[^{}]'.format(valid_subdir))
-            valid_host = 'a-zA-Z0-9_\./'
+            valid_host = 'a-zA-Z0-9_\./'  # noqa
             invalid_host = re.compile('[^{}]'.format(valid_host))
 
             if subdir is None and host is not None:
@@ -400,7 +400,9 @@ class SecretsEnvironment(object):
     def read_secrets_descriptions(self):
         """Load the descriptions of groups of secrets from a .d directory"""
         groups_dir = self.descriptions_path()
-        if os.path.exists(groups_dir):
+        if not os.path.exists(groups_dir):
+            self.LOG.info('secrets descriptions directory not found')
+        else:
             # Ignore .order file and any other non-YAML file extensions
             extensions = ['yml', 'yaml']
             file_names = [fn for fn in os.listdir(groups_dir)
@@ -408,35 +410,35 @@ class SecretsEnvironment(object):
             self._groups = [os.path.splitext(fn) for fn in file_names]
             self.LOG.debug('reading secrets descriptions from {}'.format(
                 groups_dir))
-            try:
-                # Iterate over files in directory, loading them into
-                # dictionaries as dictionary keyed on group name.
-                for fname in file_names:
-                    group = os.path.splitext(fname)[0]
-                    data = self.get_descriptions(
-                        os.path.join(groups_dir, fname))
-                    if data is not None:
-                        self._descriptions[group] = data
-                        # Dynamically create maps keyed on variable name
-                        # for simpler lookups. (See the get_prompt() method
-                        # for an example.)
-                        for d in data:
-                            for k, v in d.items():
-                                try:
-                                    # Add to existing map
-                                    v = v if v != d['Variable'] else None
-                                    getattr(self, k)[d['Variable']] = v
-                                except AttributeError:
-                                    raise RuntimeError(
-                                        '"{}" is not '.format(k) +
-                                        'a valid attribute')
-                    else:
-                        raise RuntimeError('descriptions for group ' +
-                                           '"{}" is empty'.format(group))
-            except Exception:
+            # Iterate over files in directory, loading them into
+            # dictionaries as dictionary keyed on group name.
+            if len(file_names) == 0:
                 self.LOG.info('no secrets descriptions files found')
-        else:
-            self.LOG.info('secrets descriptions directory not found')
+            for fname in file_names:
+                group = os.path.splitext(fname)[0]
+                if os.path.splitext(group)[1] != "":
+                    raise RuntimeError('Group name cannot include ".": ' +
+                                       '{}'.format(group))
+                data = self.get_descriptions(
+                    os.path.join(groups_dir, fname))
+                if data is not None:
+                    self._descriptions[group] = data
+                    # Dynamically create maps keyed on variable name
+                    # for simpler lookups. (See the get_prompt() method
+                    # for an example.)
+                    for d in data:
+                        for k, v in d.items():
+                            try:
+                                # Add to existing map
+                                v = v if v != d['Variable'] else None
+                                getattr(self, k)[d['Variable']] = v
+                            except AttributeError:
+                                raise RuntimeError(
+                                    '"{}" is not '.format(k) +
+                                    'a valid attribute')
+                else:
+                    raise RuntimeError('descriptions for group ' +
+                                       '"{}" is empty'.format(group))
 
     def descriptions(self):
         return self._descriptions
@@ -565,7 +567,7 @@ def generate_crypt6(unique=False, password=None, salt=None):
     """Generate a crypt() style SHA512 ("$6$") digest"""
     try:  # Python 3
         import crypt
-    except ModuleNotFoundError as e:
+    except ModuleNotFoundError as e:  # noqa
         raise
     if password is None:
         raise RuntimeError('generate_crypt6(): "password" is not defined')
@@ -638,6 +640,8 @@ class SecretsShow(Lister):
 
     LOG = logging.getLogger(__name__)
 
+    # Note: Not totally DRY. Replicates some logic from SecretsDescribe()
+
     def get_parser(self, prog_name):
         parser = super(SecretsShow, self).get_parser(prog_name)
         # Sorry for the double-negative, but it works better
@@ -659,6 +663,13 @@ class SecretsShow(Lister):
             default=False,
             help="Arguments are groups to list (default: False)"
         )
+        parser.add_argument(
+            '-p', '--prompts',
+            dest='args_prompts',
+            action="store_true",
+            default=False,
+            help="Include prompts (default: False)"
+        )
         parser.add_argument('args', nargs='*', default=None)
         return parser
 
@@ -668,10 +679,17 @@ class SecretsShow(Lister):
         self.app.secrets.read_secrets_and_descriptions()
         variables = []
         if parsed_args.args_group:
+            if not len(parsed_args.args):
+                raise RuntimeError('No group specified')
             for g in parsed_args.args:
-                variables.extend(
-                    [v for v in self.app.secrets.get_items_from_group(g)]
-                 )
+                try:
+                    variables.extend(
+                        [v for v
+                         in self.app.secrets.get_items_from_group(g)]
+                    )
+                except KeyError as e:
+                    raise RuntimeError('Group {} '.format(str(e)) +
+                                       'does not exist')
         else:
             variables = parsed_args.args \
                 if len(parsed_args.args) > 0 \
@@ -693,14 +711,61 @@ class SecretsDescribe(Lister):
 
     LOG = logging.getLogger(__name__)
 
+    # Note: Not totally DRY. Replicates some logic from SecretsShow()
+
     def get_parser(self, prog_name):
         parser = super(SecretsDescribe, self).get_parser(prog_name)
+        what = parser.add_mutually_exclusive_group(required=False)
+        what.add_argument(
+            '-g', '--group',
+            dest='args_group',
+            action="store_true",
+            default=False,
+            help="Arguments are groups to list (default: False)"
+        )
+        what.add_argument(
+            '-t', '--types',
+            dest='types',
+            action="store_true",
+            default=False,
+            help="Describe types (default: False)"
+        )
+        parser.add_argument('args', nargs='*', default=None)
         return parser
 
     def take_action(self, parsed_args):
         self.LOG.debug('describing secrets')
-        columns = [k.title() for k in SECRET_TYPES[0].keys()]
-        data = [[v for k, v in i.items()] for i in SECRET_TYPES]
+        if parsed_args.types:
+            columns = [k.title() for k in SECRET_TYPES[0].keys()]
+            data = [[v for k, v in i.items()] for i in SECRET_TYPES]
+        else:
+            self.app.secrets.requires_environment()
+            self.app.secrets.read_secrets_and_descriptions()
+            variables = []
+            if parsed_args.args_group:
+                if not len(parsed_args.args):
+                    raise RuntimeError('No group specified')
+                for g in parsed_args.args:
+                    try:
+                        variables.extend(
+                            [v for v
+                                in self.app.secrets.get_items_from_group(g)]
+                        )
+                    except KeyError as e:
+                        raise RuntimeError('Group {} '.format(str(e)) +
+                                           'does not exist')
+            else:
+                variables = parsed_args.args \
+                    if len(parsed_args.args) > 0 \
+                    else [k for k, v in self.app.secrets.items()]
+            columns = ('Variable', 'Type', 'Prompt')
+            data = (
+                    [(k,
+                      self.app.secrets.get_secret_type(k),
+                      self.app.secrets.get_prompt(k))
+                        for k, v in self.app.secrets.items()
+                        if k in variables]
+            )
         return columns, data
 
 
@@ -775,32 +840,31 @@ class SecretsSet(Command):
                 self.LOG.info('no description for {}'.format(k))
                 raise RuntimeError('variable "{}" '.format(k) +
                                    'has no description')
-            if k_type == 'string':
-                if '=' not in arg:
-                    v = prompt_string(
-                        prompt=self.app.secrets.get_prompt(k),
-                        default=v)
-                    if v is None:
-                        self.LOG.info('no user input for "{}"'.format(k))
-                        return None
-                if v.startswith('@'):
-                    if v[1] == '~':
-                        _path = os.path.expanduser(v[1:])
-                    else:
-                        _path = v[1:]
-                    with open(_path, 'r') as f:
-                        v = f.read().strip()
-                elif v.startswith('!'):
-                    # >> Issue: [B603:subprocess_without_shell_equals_true] subprocess call - check for execution of untrusted input.  # noqa
-                    #    Severity: Low   Confidence: High
-                    #    Location: python_secrets/secrets.py:641
-                    p = run(v[1:].split(),
-                            stdout=PIPE,
-                            stderr=PIPE,
-                            shell=False)
-                    v = p.stdout.decode('UTF-8').strip()
-                self.LOG.debug('setting {}'.format(k))
-                self.app.secrets.set_secret(k, v)
+            if '=' not in arg:
+                v = prompt_string(
+                    prompt=self.app.secrets.get_prompt(k),
+                    default=v)
+                if v is None:
+                    self.LOG.info('no user input for "{}"'.format(k))
+                    return None
+            if v.startswith('@'):
+                if v[1] == '~':
+                    _path = os.path.expanduser(v[1:])
+                else:
+                    _path = v[1:]
+                with open(_path, 'r') as f:
+                    v = f.read().strip()
+            elif v.startswith('!'):
+                # >> Issue: [B603:subprocess_without_shell_equals_true] subprocess call - check for execution of untrusted input.  # noqa
+                #    Severity: Low   Confidence: High
+                #    Location: python_secrets/secrets.py:641
+                p = run(v[1:].split(),
+                        stdout=PIPE,
+                        stderr=PIPE,
+                        shell=False)
+                v = p.stdout.decode('UTF-8').strip()
+            self.LOG.debug('setting {}'.format(k))
+            self.app.secrets.set_secret(k, v)
 
 
 class SecretsGet(Command):
